////////////////////////////////////////////////////////////////////////////
//
//  FILTER_DEMO.PAS
//
//
//	Autor 	  : Rene Bllhoff
//  Erstellt  : Mrz 2007
//
//	(c) 2005-2007 by Rene Bllhoff - reneboellhoff (at) gmx (dot) de -
////////////////////////////////////////////////////////////////////////////
//
//  This source file may be used and distributed without
//  restriction provided that this copyright statement is not
//  removed from the file and that any derivative work contains
//  the original copyright notice and the associated disclaimer.
//
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License
//  as published by the Free Software Foundation; either version 2
//  of the License, or (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.See the GNU
//  General Public License for more details.
//
//  You should have received a copy of the GNU General Public
//  License along with this source; if not, download it	from
//  http://www.gnu.org/licenses/gpl.html
//
////////////////////////////////////////////////////////////////////////////
unit filter_demo;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, ComCtrls, ExtCtrls,


  comselect,
  fpga_com_base, codec_interface,

  fpga_dap,
  coeffs;

  
type
  TfrmMain = class(TForm)
    btReset: TButton;
    btMute: TButton;
    tbCutoff: TTrackBar;
    tbRes: TTrackBar;
    rbLP: TRadioButton;
    rbHP: TRadioButton;
    rbBP: TRadioButton;
    rbBS: TRadioButton;
    lbCut: TLabel;
    lbRes: TLabel;
    lbHdCut: TLabel;
    lbHdRes: TLabel;
    btMove: TButton;
    tmMain: TTimer;
    procedure FormCreate(Sender: TObject);
    procedure btResetClick(Sender: TObject);
    procedure tbCutoffChange(Sender: TObject);
    procedure rbLPClick(Sender: TObject);
    procedure btMuteClick(Sender: TObject);
    procedure tmMainTimer(Sender: TObject);
    procedure btMoveClick(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
  private
    { Private-Deklarationen }
  public
    fpga    : TFPGACOMBase;
    dap     : TFPGADAP;

    codec1  : TCodecInterface;
    codec2  : TCodecInterface;    

    run     : boolean;

    cutoff, res : single;
    typ : integer;

    v  : single;
    vi : single;

    procedure InitDAPProgram;
    procedure UpdateFilter;
  end;

var
  frmMain: TfrmMain;

implementation

{$R *.dfm}


procedure TfrmMain.FormCreate(Sender: TObject);
var cp, br : integer;
begin
  fpga  := TFPGACOMBase.Create;

  cp := 1;
  br := fpgacom_baudrate;

  if ComPortSelect (cp, br) then fpga.OpenPort(cp);

  codec1    := TCodecInterface.Create (fpga, 0);
  codec2    := TCodecInterface.Create (fpga, 1);
  
  dap       := TFPGADAP.Create (fpga);
  dap.Port  := $f1;

  InitDAPProgram;

  cutoff  := 80;
  res     := 0.2;
  typ     := 0;

  tbCutoff.position := trunc (1200 - (cutoff * 10));
  tbres   .position := trunc (     - (res * 100));
  rbLP.checked := true;

  UpdateFilter;

  v  := 70;
  vi := 0.3;

  run := false;

end;


procedure TfrmMain.FormDestroy(Sender: TObject);
begin
  dap.Destroy;
  fpga.destroy;
end;

procedure TfrmMain.InitDAPProgram;
begin
  dap.PMEM_StartProgram;

  dap.PMEM_Biquad ( DAP_IN_1  , DAP_OUT_SC_1, $00, $00 );
  dap.PMEM_Biquad ( DAP_IN_2  , DAP_OUT_SC_2, $08, $00 );
  dap.PMEM_Biquad ( DAP_IN_3  , DAP_OUT_SC_3, $10, $00 );
  dap.PMEM_Biquad ( DAP_IN_4  , DAP_OUT_SC_4, $18, $00 );

  dap.PMEM_EndProgram;
end;


procedure TfrmMain.UpdateFilter;
var c : TDAPBiquad;
begin
  case typ of
    0 : calccoefflp (cutoff, res, c);
    1 : calccoeffhp (cutoff, res, c);
    2 : calccoeffbp (cutoff, res, c);
    3 : calccoeffbs (cutoff, res, c);
  end;
  dap.CMEM_Biquad($00, c);
end;






procedure TfrmMain.btMoveClick(Sender: TObject);
begin
  if btMove.Caption = 'Move It !'  then btMove.Caption := 'Stop'
                                   else btMove.Caption := 'Move It !';
  run := btMove.Caption = 'Stop';
end;

procedure TfrmMain.btMuteClick(Sender: TObject);
begin
  if btMute.Caption = 'Mute'  then btMute.Caption := 'Unmute'
                              else btMute.Caption := 'Mute';
  dap.Mute(btMute.Caption = 'Unmute');
end;

procedure TfrmMain.btResetClick(Sender: TObject);
begin
  InitDAPProgram;
  UpdateFilter;
end;


procedure TfrmMain.rbLPClick(Sender: TObject);
begin
  if sender = rbLP then typ := 0;
  if sender = rbHP then typ := 1;
  if sender = rbBP then typ := 2;
  if sender = rbBS then typ := 3;
  UpdateFilter;
end;


procedure TfrmMain.tbCutoffChange(Sender: TObject);
begin
  if sender = tbCutoff then cutoff  := 120 - (tbCutoff.Position /  10);
  if sender = tbRes    then res     :=     - (tbRes.Position    / 100);

  lbCut.Caption := format ('%.1f', [cutoff]);
  lbRes.Caption := format ('%.2f', [res]);

  updatefilter;
end;

procedure TfrmMain.tmMainTimer(Sender: TObject);
begin
  if run then
  begin
    if v > 110 then vi := -vi;
    if v <  10 then vi := -vi;
    v := v + vi;

    tbCutoff.Position := trunc (1200 - (v * 10));
  end;
end;

end.
