////////////////////////////////////////////////////////////////////////////
//
//  MIX_DEMO.PAS
//
//
//	Autor 	  : Rene Bllhoff
//  Erstellt  : Mrz 2007
//
//	(c) 2005-2007 by Rene Bllhoff - reneboellhoff (at) gmx (dot) de -
////////////////////////////////////////////////////////////////////////////
//
//  This source file may be used and distributed without
//  restriction provided that this copyright statement is not
//  removed from the file and that any derivative work contains
//  the original copyright notice and the associated disclaimer.
//
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License
//  as published by the Free Software Foundation; either version 2
//  of the License, or (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.See the GNU
//  General Public License for more details.
//
//  You should have received a copy of the GNU General Public
//  License along with this source; if not, download it	from
//  http://www.gnu.org/licenses/gpl.html
//
////////////////////////////////////////////////////////////////////////////
unit mix_demo;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, ExtCtrls, ComCtrls, math,


  comselect,
  fpga_com_base, codec_interface,

  fpga_dap,
  coeffs;

type
  TfrmMain = class(TForm)
    btReset: TButton;
    btMute: TButton;
    tbCh1: TTrackBar;
    tbCh2: TTrackBar;
    tbCh3: TTrackBar;
    tbCh4: TTrackBar;
    tbSum: TTrackBar;
    tbMon: TTrackBar;
    rbMon1: TRadioButton;
    rbMon2: TRadioButton;
    rbMon3: TRadioButton;
    rbMon4: TRadioButton;
    rbMonMix: TRadioButton;
    lbHdCh1: TLabel;
    lbHdCh2: TLabel;
    lbHdCh3: TLabel;
    lbHdCh4: TLabel;
    lbHdSum: TLabel;
    lbMonitor: TLabel;
    lbCh1: TLabel;
    lbCh2: TLabel;
    lbCh3: TLabel;
    lbCh4: TLabel;
    lbSum: TLabel;
    lbMon: TLabel;
    procedure btResetClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure tbCh1Change(Sender: TObject);
    procedure rbMon1Click(Sender: TObject);
    procedure btMuteClick(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
  private
    { Private-Deklarationen }
  public
    { Public-Deklarationen }

    fpga    : TFPGACOMBase;
    dap     : TFPGADAP;

    codec1  : TCodecInterface;
    codec2  : TCodecInterface;

    run     : boolean;


    channels : array [0 .. 3] of single;

    mixlevel : single;
    monlevel : single;

    monsel  : integer;

    otx : cardinal;

    procedure InitDAPProgram;
    procedure UpdateMixer;

    procedure SetCoeffOnOff (adr : byte; onoff : boolean);
    procedure SetCoeffVolume (adr : byte; val : single);

  end;

var
  frmMain: TfrmMain;

implementation

{$R *.dfm}


procedure TfrmMain.FormCreate(Sender: TObject);
var cp, br : integer;
begin
  fpga  := TFPGACOMBase.Create;

  cp := 1;
  br := fpgacom_baudrate;

  if ComPortSelect (cp, br) then fpga.OpenPort(cp);

  codec1    := TCodecInterface.Create (fpga, 0);
  codec2    := TCodecInterface.Create (fpga, 1);

  dap       := TFPGADAP.Create (fpga);
  dap.Port  := $f1;

  InitDAPProgram;

  tbCh1.Position := trunc (1000 * (1 - ( 60 / 72 )  ));
  tbCh2.Position := trunc (1000 * (1 - ( 60 / 72 )  ));
  tbCh3.Position := trunc (1000 * (1 - ( 60 / 72 )  ));
  tbCh4.Position := trunc (1000 * (1 - ( 60 / 72 )  ));
  tbSum.Position := trunc (1000 * (1 - ( 60 / 72 )  ));
  tbMon.Position := trunc (1000 * (1 - ( 60 / 72 )  ));

  UpdateMixer;


  otx := gettickcount;
end;



procedure TfrmMain.FormDestroy(Sender: TObject);
begin
  dap.Destroy;
  fpga.destroy;
end;

procedure TfrmMain.InitDAPProgram;
begin
  // cmem map :
  // 0x00  channel 1 level
  // 0x01  channel 2 level
  // 0x02  channel 3 level
  // 0x03  channel 4 level
  // 0x04  mon select channel 1
  // 0x05  mon select channel 2
  // 0x06  mon select channel 3
  // 0x07  mon select channel 4
  // 0x08  mon select sum
  // 0x09  mix out level L
  // 0x0a  mix out level R
  // 0x0b  mon out level

  dap.PMEM_StartProgram;


  dap.PMEM_Volume ( DAP_IN_1 , DAP_OUT_SC_1,      $00 );
  dap.PMEM_Store  ( DAP_IN_SC, DAP_OUT_SC_1, $00      );

  dap.PMEM_Volume ( DAP_IN_2 , DAP_OUT_SC_2,      $01 );
  dap.PMEM_Store  ( DAP_IN_SC, DAP_OUT_SC_2, $01      );

  dap.PMEM_Volume ( DAP_IN_3 , DAP_OUT_SC_3,      $02 );
  dap.PMEM_Store  ( DAP_IN_SC, DAP_OUT_SC_3, $02      );

  dap.PMEM_Volume ( DAP_IN_4 , DAP_OUT_SC_4,      $03 );
  dap.PMEM_Store  ( DAP_IN_SC, DAP_OUT_SC_4, $03      );



  dap.PMEM_Volume ( DAP_IN_1 , DAP_OUT_SC_1,      $04 );
  dap.PMEM_Store  ( DAP_IN_SC, DAP_OUT_SC_1, $04      );

  dap.PMEM_Volume ( DAP_IN_2 , DAP_OUT_SC_2,      $05 );
  dap.PMEM_Store  ( DAP_IN_SC, DAP_OUT_SC_2, $05      );

  dap.PMEM_Volume ( DAP_IN_3 , DAP_OUT_SC_3,      $06 );
  dap.PMEM_Store  ( DAP_IN_SC, DAP_OUT_SC_3, $06      );

  dap.PMEM_Volume ( DAP_IN_4 , DAP_OUT_SC_4,      $07 );
  dap.PMEM_Store  ( DAP_IN_SC, DAP_OUT_SC_4, $07      );



  dap.PMEM_ClrSum (            DAP_OUT_SC_1           );
  dap.PMEM_AddMem (            DAP_OUT_SC_1,      $00 );
  dap.PMEM_AddMem (            DAP_OUT_SC_1,      $01 );
  dap.PMEM_AddMem (            DAP_OUT_SC_1,      $02 );
  dap.PMEM_AddMem (            DAP_OUT_SC_1,      $03 );
  dap.PMEM_Nop    ( DAP_IN_SC, DAP_OUT_SC_1);
  dap.PMEM_Store  ( DAP_IN_SC, DAP_OUT_SC_1, $08      );


  dap.PMEM_Load   (            DAP_OUT_SC_1,      $08 );
  dap.PMEM_Volume ( DAP_IN_SC, DAP_OUT_SC_1,      $08 );
  dap.PMEM_Store  ( DAP_IN_SC, DAP_OUT_SC_1, $09      );


  dap.PMEM_ClrSum (            DAP_OUT_SC_1           );
  dap.PMEM_AddMem (            DAP_OUT_SC_1,      $04 );
  dap.PMEM_AddMem (            DAP_OUT_SC_1,      $05 );
  dap.PMEM_AddMem (            DAP_OUT_SC_1,      $06 );
  dap.PMEM_AddMem (            DAP_OUT_SC_1,      $07 );
  dap.PMEM_AddMem (            DAP_OUT_SC_1,      $09 );
  dap.PMEM_Nop    ( DAP_IN_SC, DAP_OUT_SC_1);    
  dap.PMEM_Store  ( DAP_IN_SC, DAP_OUT_SC_2, $0a      );



  dap.PMEM_Load   (            DAP_OUT_SC_1,      $08 );
  dap.PMEM_Volume ( DAP_IN_SC, DAP_OUT_SC_1,      $09 );

  dap.PMEM_Load   (            DAP_OUT_SC_2,      $08 );
  dap.PMEM_Volume ( DAP_IN_SC, DAP_OUT_SC_2,      $0a );

  dap.PMEM_Load   (            DAP_OUT_SC_3,      $0a );
  dap.PMEM_Volume ( DAP_IN_SC, DAP_OUT_SC_3,      $0b );

  dap.PMEM_Load   (            DAP_OUT_SC_4,      $0a );
  dap.PMEM_Volume ( DAP_IN_SC, DAP_OUT_SC_4,      $0b );


  dap.PMEM_EndProgram;


end;



procedure TfrmMain.UpdateMixer;
begin
  SetCoeffVolume ($00, (channels [0] * 72) - 60);
  SetCoeffVolume ($01, (channels [1] * 72) - 60);
  SetCoeffVolume ($02, (channels [2] * 72) - 60);
  SetCoeffVolume ($03, (channels [3] * 72) - 60);

  SetCoeffOnOff  ($04, monsel = 1);
  SetCoeffOnOff  ($05, monsel = 2);
  SetCoeffOnOff  ($06, monsel = 3);
  SetCoeffOnOff  ($07, monsel = 4);
  SetCoeffOnOff  ($08, monsel = 5);


  SetCoeffVolume ($09, (mixlevel * 72) - 60);
  SetCoeffVolume ($0a, (mixlevel * 72) - 60);

  SetCoeffVolume ($0b, (monlevel * 72) - 60);
end;


procedure TfrmMain.SetCoeffOnOff (adr : byte; onoff : boolean);
begin
  if onoff  then dap.CMEM_Longint(adr, $01000000)
            else dap.CMEM_Longint(adr, $00000000);
end;

procedure TfrmMain.SetCoeffVolume (adr : byte; val : single);
begin
  val := max (-60, min (12, val));
  val := power (10, val / 20);
  dap.CMEM_Longint(adr, round (val * $01000000));
end;







procedure TfrmMain.rbMon1Click(Sender: TObject);
begin
  if sender = rbMon1    then monsel := 1;
  if sender = rbMon2    then monsel := 2;
  if sender = rbMon3    then monsel := 3;
  if sender = rbMon4    then monsel := 4;
  if sender = rbMonMix  then monsel := 5;

  UpdateMixer;
end;


procedure TfrmMain.btMuteClick(Sender: TObject);
begin
  if btMute.Caption = 'Mute'  then btMute.Caption := 'Unmute'
                              else btMute.Caption := 'Mute';
  dap.Mute(btMute.Caption = 'Unmute');
end;

procedure TfrmMain.btResetClick(Sender: TObject);
begin
  InitDAPProgram;
end;


procedure TfrmMain.tbCh1Change(Sender: TObject);
begin
  if abs (gettickcount - otx) > 50  then
  begin
    if sender = tbCh1 then channels [0] :=  1 - (tbCh1.Position / 1000);
    if sender = tbCh2 then channels [1] :=  1 - (tbCh2.Position / 1000);
    if sender = tbCh3 then channels [2] :=  1 - (tbCh3.Position / 1000);
    if sender = tbCh4 then channels [3] :=  1 - (tbCh4.Position / 1000);
    if sender = tbSum then mixlevel     :=  1 - (tbSum.Position / 1000);
    if sender = tbMon then monlevel     :=  1 - (tbMon.Position / 1000);

    lbCh1.caption := format ('%.1f dB', [ (channels [0] * 72 - 60) ]);
    lbCh2.caption := format ('%.1f dB', [ (channels [1] * 72 - 60) ]);
    lbCh3.caption := format ('%.1f dB', [ (channels [2] * 72 - 60) ]);
    lbCh4.caption := format ('%.1f dB', [ (channels [3] * 72 - 60) ]);

    lbSum.caption := format ('%.1f dB', [ (mixlevel * 72 - 60) ]);

    lbMon.caption := format ('%.1f dB', [ (monlevel * 72 - 60) ]);

    UpdateMixer;
  end;
end;





end.
