////////////////////////////////////////////////////////////////////////////
//
//  SYNTH_DEMO_UTIL.PAS
//
//
//	Autor 	  : Rene Bllhoff
//  Erstellt  : April 2007
//
//	(c) 2005-2007 by Rene Bllhoff - reneboellhoff (at) gmx (dot) de -
////////////////////////////////////////////////////////////////////////////
//
//  This source file may be used and distributed without
//  restriction provided that this copyright statement is not
//  removed from the file and that any derivative work contains
//  the original copyright notice and the associated disclaimer.
//
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License
//  as published by the Free Software Foundation; either version 2
//  of the License, or (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.See the GNU
//  General Public License for more details.
//
//  You should have received a copy of the GNU General Public
//  License along with this source; if not, download it	from
//  http://www.gnu.org/licenses/gpl.html
//
////////////////////////////////////////////////////////////////////////////
unit synth_demo_util;

interface

uses windows, sysutils, classes, math, 

     fpga_dap,

     coeffs, synthtools, strfuncs;


type TSynthCtrlCore = class
     protected
       // main ctrl
       fbasepitch : single;    // globales pitch
       fvolume    : single;
       fcv_source : byte;      // 0 = direkt (gate = 1) 1 = ext-gate 2 = sequencer


       fseqpitchbase : single;

       // ext gate
       fext_gate    : single;     // "externes" gate (button)
       fext_cv      : single;     // "externes" cv

       // direkt
       fdirectlevel : single;

       // sequencer
       seqcnt      : byte;
       seqcntmax   : byte;
       seq_ptr     : byte;

       fseqpitch : array [0 .. 15] of single;
       fseqgate  : array [0 .. 15] of single;
       fseqctrl  : array [0 .. 15] of single;

       fcspitch, fcsgate, fcsctrl : single;


       // filter
       fcutoffbase     : single;  // direkt bei direkt und ext gate, als base-offset bei sequencer
       fcutoff, fres   : single;  // aktuelles cutoff und aktuelle resonanz
       fftype          : byte;    // filtertyp

       focutoff, fores : single;  // alte cutoff, res, type fr optimierung
       fotype          : byte;

       // envelope
       adsr : tadsr;

       fA, fD, fS, fR : single;   // zeiten


       // oscillator
       fupdtune, fupdoscwave, fupdosclevel : boolean; // update bereich (fr optimierung)

       fores_pitch : single;

       osc1pitch : single;      // tuning (+36 .. -36)
       osc2pitch : single;
       osc3pitch : single;

       osc1level, osc1pwm, osc1wave : byte;  // osclevel (0..255) oscpwm (0..255) wave (0,1,3,2,6,10,14)
       osc2level, osc2pwm, osc2wave : byte;
       osc3level, osc3pwm, osc3wave : byte;




       procedure setseqcntmax (v : byte);
       procedure setbasepitch (v : single);

       function  getseqgate   (i : integer) : single;
       function  getseqpitch  (i : integer) : single;
       function  getseqCtrl   (i : integer) : single;
       procedure setseqgate   (i : integer; v : single);
       procedure setseqpitch  (i : integer; v : single);
       procedure setseqCtrl   (i : integer; v : single);


       procedure setosctune   (i : integer; v : single);
       function  getosctune   (i : integer) : single;
       procedure setoscwave   (i : integer; v : byte);
       function  getoscwave   (i : integer) : byte;
       procedure setoscpwm    (i : integer; v : single);
       function  getoscpwm    (i : integer) : single;
       procedure setosclev    (i : integer; v : single);
       function  getosclev    (i : integer) : single;


       procedure setcutoff    (v : single);
       procedure setres       (v : single);
       procedure settype      (v : byte);


       procedure setenva      (v : single);
       procedure setenvd      (v : single);
       procedure setenvs      (v : single);
       procedure setenvr      (v : single);



       procedure Init;

       procedure DoEnvelope;
       procedure DoSeqencer;
       procedure DoFilter;
       procedure DoOscillators;


    public

       dap : TFPGADAP;


       constructor Create;
       destructor Destroy;

       procedure DoSynthCore;

       procedure LoadSynth (x : ansistring);
       function  SaveSynth : ansistring;

       procedure LoadSequencer (x : ansistring);
       function  SaveSequencer : ansistring;


       property CV_Source               : byte    read fcv_source write fcv_source;
       property SeqPrescaler            : byte    read seqcntmax  write setseqcntmax;
       property BasePitch               : single  read fbasepitch write setbasepitch;

       property Ext_Gate                : single  read fext_gate  write fext_gate;
       property Ext_CV                  : single  read fext_cv    write fext_cv;


       property OscTune  [i : integer]  : single  read getosctune write setosctune;
       property OscWave  [i : integer]  : byte    read getoscwave write setoscwave;
       property OscPWM   [i : integer]  : single  read getoscpwm  write setoscpwm;
       property OscLevel [i : integer]  : single  read getosclev  write setosclev;

       property FilterCutoff            : single  read fcutoff    write setcutoff;
       property FilterRes               : single  read fres       write setres;
       property FilterType              : byte    read fftype     write settype;


       property EnvA                    : single  read fa         write setenva;
       property EnvD                    : single  read fd         write setenvd;
       property EnvS                    : single  read fs         write setenvs;
       property EnvR                    : single  read fr         write setenvr;


       property SeqencerGate  [i : integer] : single read getseqGate  write setseqgate;
       property SeqencerPitch [i : integer] : single read getseqPitch write setseqpitch;
       property SeqencerCtrl  [i : integer] : single read getseqCtrl  write setseqCtrl;



     end;




const UPD_OSC_ALL       =  0;
const UPD_OSC_PITCH     =  1;


const UPD_OSC_WAVE1     =  2;
const UPD_OSC_WAVE2     =  3;
const UPD_OSC_WAVE3     =  4;

const UPD_OSC_FRQ1      =  5;
const UPD_OSC_FRQ2      =  6;
const UPD_OSC_FRQ3      =  7;

const UPD_OSC_LEVEL1    =  8;
const UPD_OSC_LEVEL2    =  9;
const UPD_OSC_LEVEL3    = 10;



implementation

constructor TSynthCtrlCore.Create;
begin
  inherited create;
  
  adsr := tadsr.Create (0.01);

  init;
end;

destructor TSynthCtrlCore.Destroy;
begin
  adsr.Destroy;
  inherited;
end;


procedure TSynthCtrlCore.Init;
begin

  fbasepitch := 33;
  fvolume    := 0.7;
  fcv_source := 1;


  fseqpitchbase := 33;

  fext_gate    := 0.0;
  fext_cv      := 0.0;

  fdirectlevel := 0.7;



  seqcnt := 0;
  seqcntmax := 8;
  seq_ptr := 0;





  fseqpitch [00] := -12.0;
  fseqpitch [01] := 0.0;
  fseqpitch [02] := 0.0;
  fseqpitch [03] := 0.0;
  fseqpitch [04] := 0.0;
  fseqpitch [05] := 0.0;
  fseqpitch [06] := 0.0;
  fseqpitch [07] := 0.0;
  fseqpitch [08] := 0.0;
  fseqpitch [09] := 0.0;
  fseqpitch [10] := 0.0;
  fseqpitch [11] := 0.0;
  fseqpitch [12] := 10.0;
  fseqpitch [13] := 0.0;
  fseqpitch [14] := 12.0;
  fseqpitch [15] := 0.0;

  fseqgate [00]  := 1.0;
  fseqgate [01]  := 0.0;
  fseqgate [02]  := 0.0;
  fseqgate [03]  := 0.7;
  fseqgate [04]  := 0.0;
  fseqgate [05]  := 0.0;
  fseqgate [06]  := 1.0;
  fseqgate [07]  := 0.0;
  fseqgate [08]  := 0.0;
  fseqgate [09]  := 0.8;
  fseqgate [10]  := 0.0;
  fseqgate [11]  := 0.0;
  fseqgate [12]  := 0.7;
  fseqgate [13]  := 0.0;
  fseqgate [14]  := 1.0;
  fseqgate [15]  := 0.0;

  fseqctrl [00]  := 0.5;
  fseqctrl [01]  := 0.3;
  fseqctrl [02]  := 0.1;
  fseqctrl [03]  := 0.2;
  fseqctrl [04]  := 0.7;
  fseqctrl [05]  := 0.0;
  fseqctrl [06]  := 0.15;
  fseqctrl [07]  := 0.0;
  fseqctrl [08]  := 0.1;
  fseqctrl [09]  := 0.15;
  fseqctrl [10]  := 0.6;
  fseqctrl [11]  := 0.0;
  fseqctrl [12]  := 0.25;
  fseqctrl [13]  := 0.2;
  fseqctrl [14]  := 0.4;
  fseqctrl [15]  := 0.35;

end;




procedure TSynthCtrlCore.DoFilter;
var c : TDAPBiquad;
begin
  case fcv_source of
    0 ,
    1 : fcutoff := fcutoffbase;
    2 : fcutoff := fcutoffbase + (fcsctrl * 100);
  end;

  if (fcutoff <> focutoff) or (fores <> fres) or (fotype <> fftype) then
  begin
    focutoff := fcutoff;
    fores    := fres;
    fotype   := fftype;

    case fftype of
      0 : calccoeffBypass (c);
      1 : calccoeffLP     (fcutoff, fres, c);
      2 : calccoeffHP     (fcutoff, fres, c);
      3 : calccoeffBP     (fcutoff, fres, c);
      4 : calccoeffBS     (fcutoff, fres, c);
    end;
    dap.CMEM_Biquad ($04, c);
  end;
end;


procedure TSynthCtrlCore.DoOscillators;
var pr1, pr2, pr3 : dword;
    lr1, lr2, lr3 : dword;
    fres_pitch : single;
begin
  case fcv_source of
    0 : fres_pitch := 33;
    1 : fres_pitch := fbasepitch + fext_cv;
    2 : fres_pitch := fbasepitch + fcspitch;
  end;

  if fores_pitch <> fres_pitch then begin fupdtune := true; fores_pitch := fres_pitch; end;
  

  if fupdtune then
  begin
    fupdtune := false;

    pr1 := trunc ( (p2f (fres_pitch + osc1pitch) / 48000.0 ) * $ffffffff);
    pr2 := trunc ( (p2f (fres_pitch + osc2pitch) / 48000.0 ) * $ffffffff);
    pr3 := trunc ( (p2f (fres_pitch + osc3pitch) / 48000.0 ) * $ffffffff);

    dap.DMEM_Osc   ($00, pr1);
    dap.DMEM_Osc   ($04, pr2);
    dap.DMEM_Osc   ($08, pr3);
  end;


  if fupdoscwave then
  begin
    fupdoscwave := false;

    dap.DMEM_Shape ($02, osc1wave, osc1pwm);
    dap.DMEM_Shape ($06, osc2wave, osc2pwm);
    dap.DMEM_Shape ($0a, osc3wave, osc3pwm);
  end;

  if fupdosclevel then
  begin
    fupdosclevel := false;

    lr1 := osc1level shl 16;
    lr2 := osc2level shl 16;
    lr3 := osc3level shl 16;

    dap.CMEM_Longint ($00, lr1);
    dap.CMEM_Longint ($01, lr2);
    dap.CMEM_Longint ($02, lr3);
  end;

end;



procedure TSynthCtrlCore.DoEnvelope;
begin
  case fcv_source of
    0 : dap.CMEM_Longint ($03, round (fdirectlevel * $01000000));
    1 : dap.CMEM_Longint ($03, adsr.DoEnv (fext_gate) shl 8);
    2 : dap.CMEM_Longint ($03, adsr.DoEnv (fcsgate) shl 8);
  end;
end;

procedure TSynthCtrlCore.DoSeqencer;
begin
  seqcnt := seqcnt + 1;
  if seqcnt = 0 then
    begin
      fcsgate  := fseqgate  [seq_ptr];
      fcspitch := trunc (fseqpitch [seq_ptr]);
      fcsctrl  := fseqctrl  [seq_ptr];
    end
  else if seqcnt = seqcntmax div 2 then
    begin
      fcsgate := 0;
    end
  else if seqcnt = seqcntmax then
    begin
      seq_ptr := seq_ptr + 1;
      if seq_ptr = 16 then seq_ptr := 0;
      seqcnt := 255;
    end;
end;

procedure TSynthCtrlCore.DoSynthCore;
begin
  DoSeqencer;
  DoOscillators;
  DoFilter;
  DoEnvelope;
end;



















procedure TSynthCtrlCore.setseqcntmax (v : byte);
begin
  if v in [4..10] then
  begin
    seqcnt    := 0;
    seqcntmax := v;
  end;
end;

procedure TSynthCtrlCore.setbasepitch (v : single);
begin
  v := min (80, max (0, v));
  fbasepitch := v;
  fupdtune := true;
end;



procedure TSynthCtrlCore.setoscwave   (i : integer; v : byte);
var wave : byte;
begin
  wave := 0;

  if v = 0 then wave :=  0;
  if v = 1 then wave :=  1;
  if v = 2 then wave :=  3;
  if v = 3 then wave :=  2;
  if v = 4 then wave :=  6;
  if v = 5 then wave := 10;
  if v = 6 then wave := 14;

  if i = 1  then begin osc1wave := wave; fupdoscwave := true; end;
  if i = 2  then begin osc2wave := wave; fupdoscwave := true; end;
  if i = 3  then begin osc3wave := wave; fupdoscwave := true; end;
end;

function  TSynthCtrlCore.getoscwave  (i : integer) : byte;
var src : byte;
begin
  src := 0;
  if i = 1  then src := osc1wave;
  if i = 2  then src := osc2wave;
  if i = 3  then src := osc3wave;

  if src =  0 then result := 0;
  if src =  1 then result := 1;
  if src =  3 then result := 2;
  if src =  2 then result := 3;
  if src =  6 then result := 4;
  if src = 10 then result := 5;
  if src = 14 then result := 6;
end;



procedure TSynthCtrlCore.setosctune   (i : integer; v : single);
begin
  v := min (36, max (-36, v));
  if i = 1 then begin osc1pitch := v; fupdtune := true; end;
  if i = 2 then begin osc2pitch := v; fupdtune := true; end;
  if i = 3 then begin osc3pitch := v; fupdtune := true; end;
end;

function  TSynthCtrlCore.getosctune   (i : integer) : single;
begin
  result := 0;
  if i = 1 then result := osc1pitch;
  if i = 2 then result := osc2pitch;
  if i = 3 then result := osc3pitch;
end;


procedure TSynthCtrlCore.setoscpwm    (i : integer; v : single);
begin
  v := min (1, max (0, v));
  if i = 1  then begin osc1pwm  := trunc (v * 255); fupdoscwave := true; end;
  if i = 2  then begin osc2pwm  := trunc (v * 255); fupdoscwave := true; end;
  if i = 3  then begin osc3pwm  := trunc (v * 255); fupdoscwave := true; end;
end;

function  TSynthCtrlCore.getoscpwm    (i : integer) : single;
begin
  result := 0;
  if i = 1 then result := osc1pwm / 255; 
  if i = 2 then result := osc2pwm / 255; 
  if i = 3 then result := osc3pwm / 255;
end;


procedure TSynthCtrlCore.setosclev    (i : integer; v : single);
begin
  v := min (1, max (0, v));
  if i = 1 then begin osc1level := trunc (v * 255); fupdosclevel := true; end;
  if i = 2 then begin osc2level := trunc (v * 255); fupdosclevel := true; end;
  if i = 3 then begin osc3level := trunc (v * 255); fupdosclevel := true; end;
end;


function  TSynthCtrlCore.getosclev    (i : integer) : single;
begin
  result := 0;
  if i = 1 then result := osc1level / 255;
  if i = 2 then result := osc2level / 255;
  if i = 3 then result := osc3level / 255;
end;













procedure TSynthCtrlCore.setcutoff    (v : single);
begin
  v := min (120, max (0, v));
  fcutoffbase := v;
end;

procedure TSynthCtrlCore.setres       (v : single);
begin
  v := min (1, max (-1, v));
  fres := v;
end;

procedure TSynthCtrlCore.settype      (v : byte);
begin
  if v in [0 .. 4] then fftype := v;
end;








procedure TSynthCtrlCore.setenva      (v : single);
begin
  fa := v;
  adsr.SetEnv (fa, fd, fs, fr);
end;

procedure TSynthCtrlCore.setenvd      (v : single);
begin
  fd := v;
  adsr.SetEnv (fa, fd, fs, fr);
end;

procedure TSynthCtrlCore.setenvs      (v : single);
begin
  fs := v;
  adsr.SetEnv (fa, fd, fs, fr);
end;

procedure TSynthCtrlCore.setenvr      (v : single);
begin
  fr := v;
  adsr.SetEnv (fa, fd, fs, fr);
end;







procedure TSynthCtrlCore.setseqgate  (i : integer; v : single);
begin
  if i in [0 .. 15] then
  begin
    v := min (1, max (0, v));
    fseqgate [i] := v;
  end;
end;

procedure TSynthCtrlCore.setseqpitch (i : integer; v : single);
begin
  if i in [0 .. 15] then
  begin
    v := min (36, max (-36, v));
    fseqpitch [i] := v;
  end;
end;

procedure TSynthCtrlCore.setseqCtrl  (i : integer; v : single);
begin
  if i in [0 .. 15] then
  begin
    v := min (1, max (0, v));
    fseqctrl [i] := v;
  end;
end;

function  TSynthCtrlCore.getseqgate  (i : integer) : single;
begin
  result := 0.0;
  if i in [0 .. 15] then
  begin
    result := fseqgate[i];
  end;
end;

function  TSynthCtrlCore.getseqpitch (i : integer) : single;
begin
  result := 0.0;
  if i in [0 .. 15] then
  begin
    result := fseqpitch[i];
  end;
end;

function  TSynthCtrlCore.getseqCtrl  (i : integer) : single;
begin
  result := 0.0;
  if i in [0 .. 15] then
  begin
    result := fseqctrl[i];
  end;
end;





procedure TSynthCtrlCore.LoadSynth (x : ansistring);
var line, item, value : ansistring;
    vbyte : byte;
    vsingle : single;
    vbe, vse : integer;
begin
  repeat
    line := getuntilwosp (x, ';', x);
    item := lowercase (getuntil (x, '=', value));
    val (value, vbyte  , vbe);
    val (value, vsingle, vse);

    if (item <> '') and ( (vbe = 0) or (vse = 0) ) then
    begin
      if item = 'osc1pitch' then OscTune  [1] := vsingle;
      if item = 'osc2pitch' then OscTune  [2] := vsingle;
      if item = 'osc3pitch' then OscTune  [3] := vsingle;
      if item = 'osc1wave'  then OscWave  [1] := vbyte;
      if item = 'osc2wave'  then OscWave  [2] := vbyte;
      if item = 'osc3wave'  then OscWave  [3] := vbyte;
      if item = 'osc1pwm'   then OscPwm   [1] := vsingle;
      if item = 'osc2pwm'   then OscPwm   [2] := vsingle;
      if item = 'osc3pwm'   then OscPwm   [3] := vsingle;
      if item = 'osc1level' then OscLevel [1] := vsingle;
      if item = 'osc2level' then OscLevel [2] := vsingle;
      if item = 'osc3level' then OscLevel [3] := vsingle;

      if item = 'cutoff'    then FilterCutoff := vSingle;
      if item = 'res'       then FilterRes    := vsingle;
      if item = 'type'      then FilterType   := vbyte;

      if item = 'attack'    then EnvA         := vsingle;
      if item = 'decay'     then EnvD         := vsingle;
      if item = 'sustain'   then EnvS         := vsingle;
      if item = 'release'   then EnvR         := vsingle;

      if item = ''          then

      if item = ''          then      
    end;
  until item = '';
end;

function  TSynthCtrlCore.SaveSynth : ansistring;
begin
  result := '';
  result := result + format ('osc1pitch=%.f;' , [OscTune  [1]]);
  result := result + format ('osc2pitch=%.f;' , [OscTune  [2]]);
  result := result + format ('osc3pitch=%.f;' , [OscTune  [3]]);
  result := result + format ('osc1wave =%.f;' , [OscWave  [1]]);
  result := result + format ('osc2wave =%.f;' , [OscWave  [2]]);
  result := result + format ('osc3wave =%.f;' , [OscWave  [3]]);
  result := result + format ('osc1pwm  =%.f;' , [OscPwm   [1]]);
  result := result + format ('osc2pwm  =%.f;' , [OscPwm   [2]]);
  result := result + format ('osc3pwm  =%.f;' , [OscPwm   [3]]);
  result := result + format ('osc1level=%.f;' , [OscLevel [1]]);
  result := result + format ('osc2level=%.f;' , [OscLevel [2]]);
  result := result + format ('osc3level=%.f;' , [OscLevel [3]]);

  result := result + format ('cutoff   =%.f;' , [FilterCutoff]);
  result := result + format ('res      =%.f;' , [FilterRes   ]);
  result := result + format ('type     =%.f;' , [FilterType  ]);

  result := result + format ('attack   =%.f;' , [EnvA        ]);
  result := result + format ('decay    =%.f;' , [EnvD        ]);
  result := result + format ('sustain  =%.f;' , [EnvS        ]);
  result := result + format ('release  =%.f;' , [EnvR        ]);
end;


procedure TSynthCtrlCore.LoadSequencer (x : ansistring);
var tag, data, sval : ansistring;
    c, e            : integer;
    fval            : single;
begin
  repeat
    tag   := getnexttagname (x);
    data  := cuttag (x, tag);
          if tag = 'gate' then
      begin
        for c := 0 to 15 do
          begin
            sval := getuntilwosp (data, ';', data);
            val (sval, fval, e);
            if e = 0 then
              SeqencerGate [c] := fval;
          end;
      end
    else  if tag = 'pitch' then
      begin
        for c := 0 to 15 do
          begin
            sval := getuntilwosp (data, ';', data);
            val (sval, fval, e);
            if e = 0 then
              SeqencerPitch [c] := fval;
          end;
      end
    else  if tag = 'ctrl' then
      begin
        for c := 0 to 15 do
          begin
            sval := getuntilwosp (data, ';', data);
            val (sval, fval, e);
            if e = 0 then
              SeqencerCtrl [c] := fval;
          end;
      end
  until tag = '';
end;

function  TSynthCtrlCore.SaveSequencer : ansistring;
var c : integer;
    t : ansistring;
begin
  result := '';

  t := '';
  for  c := 0 to 15 do
    t := t + format ('%.f;', [SeqencerGate [c]]);
  result := result + maketag ('gate', t);

  t := '';
  for  c := 0 to 15 do
    t := t + format ('%.f;', [SeqencerPitch [c]]);
  result := result + maketag ('pitch', t);

  t := '';
  for  c := 0 to 15 do
    t := t + format ('%.f;', [SeqencerCtrl [c]]);
  result := result + maketag ('ctrl', t);
end;



end.
