library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

--  Uncomment the following lines to use the declarations that are
--  provided for instantiating Xilinx primitive components.
--library UNISIM;
--use UNISIM.VComponents.all;

entity sram_access is
		port (clk 			: in		std_logic;
					res 			: in 		std_logic;

					-- sram seite (fr 1 sram)

					sri_adr		: out 	std_logic_vector (22 downto 0);
					sri_data	: inout std_logic_vector (15 downto 0);
					sri_cs 		: out		std_logic;
					sri_adv		: out		std_logic;
					sri_oe		: out		std_logic;
					sri_ub		: out		std_logic;
					sri_lb		: out		std_logic;
					sri_we		: out		std_logic;
					sri_clk		: out		std_logic;
					sri_wait	: in  	std_logic;

					-- fpga seite

					ri_adr		: in  	std_logic_vector (22 downto 0);
					ri_din		: in    std_logic_vector (15 downto 0);
					ri_dout		: out 	std_logic_vector (15 downto 0);
					ri_rw			: in		std_logic;
					ri_trg		: in		std_logic;
					ri_rdy		: out		std_logic
				 );
end sram_access;

architecture Behavioral of sram_access is

	type ram_states is (rs_idle, 
	
											rs_write_00,
											rs_write_01, 
											rs_write_02, 
											rs_write_03,
											rs_write_04,
											rs_write_05,
											
											rs_read_00,
											rs_read_01, 
											rs_read_02,
											rs_read_03,
											rs_read_04,
											rs_read_05
											);

	signal ram_state : ram_states;
	
	signal sram_gce : std_logic;

begin

	process (clk, res, 
	         sri_Data,
	         ram_state, ri_adr, ri_din, ri_rw, ri_trg)
	begin
		sri_ub 	<= sram_gce;
		sri_lb 	<= sram_gce;
		sri_cs  <= sram_gce;
		--sri_adr <= ri_adr;
		sri_clk		<= '0';

		if res = '1' then

			ram_state <= rs_idle;			
			sri_data 	<= (others => 'Z');
			sri_adr 	<= (others => '0');
			sri_oe 		<= '1';
			sram_gce 	<= '1';
			sri_we 	 	<= '1';
			sri_adv		<= '1';

			ri_dout 	<= (others => '0');
      ri_rdy 		<= '0';

		elsif clk'event and clk = '1' then

			case ram_state is
				when rs_idle 			=>  if ri_trg = '1' then
				                        sri_adr <= ri_adr;
																sri_adv <= '0';
																
																if ri_rw = '0' then
																	ram_state <= rs_write_00;
																	sri_data 	<= (others => 'Z');
																	sram_gce 	<= '0';
																	sri_oe  	<= '1';
																	sri_we  	<= '0';
      														ri_rdy 		<= '0';
																else
																	ram_state <= rs_read_00;
																	sri_data 	<= (others => 'Z');
																	sram_gce 	<= '1';
																	sri_oe  	<= '1';
																	sri_we  	<= '1';
														      ri_rdy 		<= '0';
																end if;
																
															else
																sri_data <= (others => 'Z');
																sram_gce   	<= '1';
																sri_oe   		<= '1';
																sri_we  		<= '1';
																sri_adv			<= '1';
      													ri_rdy 			<= '1';
															end if;

				when rs_write_00 	=>	sri_data 	<= ri_din;
															
															sram_gce 	<= '0';
															sri_oe  	<= '1';
															sri_we  	<= '0';
															ri_rdy 		<= '0';
															ram_state <= rs_write_01;

				when rs_write_01 	=>  
															sram_gce 	<= '0';
															sri_oe  	<= '1';
															sri_we  	<= '0';
															ri_rdy 		<= '0';
															ram_state <= rs_write_02;
															

				when rs_write_02 	=>
															sram_gce 	<= '0';
															sri_oe  	<= '1';
															sri_we  	<= '0';
															ri_rdy 		<= '0';
															ram_state <= rs_write_03;


				when rs_write_03 	=>
															sram_gce 	<= '0';
															sri_oe  	<= '1';
															sri_we  	<= '0';
															ri_rdy 		<= '0';
															ram_state <= rs_write_04;															

				when rs_write_04 	=>
															sram_gce 	<= '1';
															sri_oe  	<= '1';
															sri_we  	<= '0';
															ri_rdy 		<= '0';
															ram_state <= rs_write_05;			

				when rs_write_05 	=>  sri_data 	<= (others => 'Z');
															sram_gce 	<= '1';
															sri_oe  	<= '1';
															sri_we  	<= '1';
															ri_rdy 		<= '1';
															ram_state <= rs_idle;		



				when rs_read_00		=>
															sri_data 	<= (others => 'Z');
															sram_gce 	<= '0';
															sri_oe  	<= '1';
															sri_we  	<= '1';
															ri_rdy 		<= '0';
															ram_state <= rs_read_01;

				when rs_read_01 	=>
                              
															sram_gce  <= '0';
															sri_oe   	<= '0';
															sri_we   	<= '1';
															ri_rdy 		<= '0';
															ram_state <= rs_read_02;

				when rs_read_02 	=>
															sram_gce  <= '0';
															sri_oe   	<= '0';
															sri_we   	<= '1';
															ri_rdy 		<= '0';
															ram_state <= rs_read_03;
															
				when rs_read_03 	=>
															sram_gce  <= '0';
															sri_oe   	<= '0';
															sri_we   	<= '1';
															ri_rdy 		<= '0';
															ram_state <= rs_read_04;


				when rs_read_04 	=>  ri_dout 	<= sri_data;
															sram_gce 	<= '0';
															sri_oe   	<= '0';
															sri_we   	<= '1';
															ri_rdy 		<= '0';
															ram_state <= rs_read_05;	
															

				when rs_read_05 	=>  --ri_dout 	<= sri_data;
															sram_gce 	<= '1';
															sri_oe   	<= '1';
															sri_we   	<= '1';
															ri_rdy 		<= '1';
															ram_state <= rs_idle;	

				when others 			=> 	ram_state <= rs_idle;
															ri_rdy 		<= '0';
			end case;

		end if;


	end process;

end Behavioral;
